@@ -1438,6 +1438,8 @@ func TestAdmitPVCStatus(t *testing.T) {
 	noExistingPods := corev1lister.NewPodLister(noExistingPodsIndex)
 	mynode := &user.DefaultInfo{Name: "system:node:mynode", Groups: []string{"system:nodes"}}
 
+	nodeExpansionFailed := api.PersistentVolumeClaimNodeResizeFailed
+
 	tests := []struct {
 		name                    string
 		resource                schema.GroupVersionResource
@@ -1452,11 +1454,11 @@ func TestAdmitPVCStatus(t *testing.T) {
 			name: "should not allow full pvc update from nodes",
 			oldObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			subresource: "",
 			newObj: makeTestPVC(
-				"", api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"", "10G", nil,
 			),
 			expectError: "is forbidden: may only update PVC status",
 		},
@@ -1464,13 +1466,13 @@ func TestAdmitPVCStatus(t *testing.T) {
 			name: "should allow capacity and condition updates, if expansion is enabled",
 			oldObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			expansionFeatureEnabled: true,
 			subresource:             "status",
 			newObj: makeTestPVC(
 				api.PersistentVolumeClaimFileSystemResizePending,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			expectError: "",
 		},
@@ -1478,13 +1480,13 @@ func TestAdmitPVCStatus(t *testing.T) {
 			name: "should not allow updates to allocatedResources with just expansion enabled",
 			oldObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			subresource:             "status",
 			expansionFeatureEnabled: true,
 			newObj: makeTestPVC(
 				api.PersistentVolumeClaimFileSystemResizePending,
-				api.PersistentVolumeClaimNoExpansionInProgress, "15G",
+				"15G", nil,
 			),
 			expectError: "is not allowed to update fields other than",
 		},
@@ -1492,14 +1494,14 @@ func TestAdmitPVCStatus(t *testing.T) {
 			name: "should allow updates to allocatedResources with expansion and recovery enabled",
 			oldObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			subresource:             "status",
 			expansionFeatureEnabled: true,
 			recoveryFeatureEnabled:  true,
 			newObj: makeTestPVC(
 				api.PersistentVolumeClaimFileSystemResizePending,
-				api.PersistentVolumeClaimNoExpansionInProgress, "15G",
+				"15G", nil,
 			),
 			expectError: "",
 		},
@@ -1507,14 +1509,14 @@ func TestAdmitPVCStatus(t *testing.T) {
 			name: "should allow updates to resizeStatus with expansion and recovery enabled",
 			oldObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNoExpansionInProgress, "10G",
+				"10G", nil,
 			),
 			subresource:             "status",
 			expansionFeatureEnabled: true,
 			recoveryFeatureEnabled:  true,
 			newObj: makeTestPVC(
 				api.PersistentVolumeClaimResizing,
-				api.PersistentVolumeClaimNodeExpansionFailed, "10G",
+				"10G", &nodeExpansionFailed,
 			),
 			expectError: "",
 		},
@@ -1545,8 +1547,8 @@ func TestAdmitPVCStatus(t *testing.T) {
 
 func makeTestPVC(
 	condition api.PersistentVolumeClaimConditionType,
-	resizeStatus api.PersistentVolumeClaimResizeStatus,
-	allocatedResources string) *api.PersistentVolumeClaim {
+	allocatedResources string,
+	resizeStatus *api.ClaimResourceStatus) *api.PersistentVolumeClaim {
 	pvc := &api.PersistentVolumeClaim{
 		Spec: api.PersistentVolumeClaimSpec{
 			VolumeName: "volume1",
@@ -1560,13 +1562,19 @@ func makeTestPVC(
 			Capacity: api.ResourceList{
 				api.ResourceStorage: resource.MustParse(allocatedResources),
 			},
-			Phase:        api.ClaimBound,
-			ResizeStatus: &resizeStatus,
+			Phase: api.ClaimBound,
 			AllocatedResources: api.ResourceList{
 				api.ResourceStorage: resource.MustParse(allocatedResources),
 			},
 		},
 	}
+	if resizeStatus != nil {
+		claimStatusMap := map[api.ResourceName]api.ClaimResourceStatus{
+			api.ResourceStorage: *resizeStatus,
+		}
+		pvc.Status.AllocatedResourceStatuses = claimStatusMap
+	}
+
 	if len(condition) > 0 {
 		pvc.Status.Conditions = []api.PersistentVolumeClaimCondition{
 			{
